"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var inversify_1 = require('inversify');
var CommandLineImpl = (function () {
    function CommandLineImpl(_nestedYargs) {
        this.nestedYargs = _nestedYargs;
        this.addEasyTableToConsole();
    }
    CommandLineImpl.prototype.init = function (options) {
        if (options === void 0) { options = {}; }
        this.nestedYargsApp = this.nestedYargs.createApp(options);
    };
    CommandLineImpl.prototype.addCommand = function (cmd) {
        var _this = this;
        cmd.aliases.forEach(function (alias) {
            var category = _this.nestedYargs.createCategory(alias, cmd.commandDesc);
            cmd.subCommands.forEach(function (subCommand) {
                subCommand.aliases.forEach(function (alias) {
                    var catCmd = _this.nestedYargs.createCommand(alias, subCommand.commandDesc, subCommand);
                    category.command(catCmd);
                });
            });
            _this.nestedYargsApp.command(category);
        });
    };
    CommandLineImpl.prototype.printTable = function (rows) {
        console.table(rows);
    };
    CommandLineImpl.prototype.exec = function (unitTestArgs) {
        if (unitTestArgs === void 0) { unitTestArgs = []; }
        this.nestedYargs.run(this.nestedYargsApp, unitTestArgs);
    };
    CommandLineImpl.prototype.addEasyTableToConsole = function () {
        if (typeof console === 'undefined') {
            throw new Error('Weird, console object is undefined');
        }
        if (typeof console.table === 'function') {
            return;
        }
        var Table = require('easy-table');
        function arrayToString(arr) {
            var t = new Table();
            arr.forEach(function (record) {
                if (typeof record === 'string' ||
                    typeof record === 'number') {
                    t.cell('item', record);
                }
                else {
                    // assume plain object
                    Object.keys(record).forEach(function (property) {
                        t.cell(property, record[property]);
                    });
                }
                t.newRow();
            });
            return t.toString();
        }
        function printTitleTable(title, arr) {
            var str = arrayToString(arr);
            var rowLength = str.indexOf('\n');
            if (rowLength > 0) {
                if (title.length > rowLength) {
                    rowLength = title.length;
                }
                console.log(title);
                var sep = '-', k = void 0, line = '';
                for (k = 0; k < rowLength; k += 1) {
                    line += sep;
                }
                console.log(line);
            }
            console.log(str);
        }
        function objectToArray(obj) {
            var keys = Object.keys(obj);
            return keys.map(function (key) {
                return {
                    key: key,
                    value: obj[key]
                };
            });
        }
        function objectToString(obj) {
            return arrayToString(objectToArray(obj));
        }
        console.table = function () {
            console.log('');
            var args = Array.prototype.slice.call(arguments);
            if (args.length === 2 &&
                typeof args[0] === 'string' &&
                Array.isArray(args[1])) {
                return printTitleTable(args[0], args[1]);
            }
            args.forEach(function (k) {
                if (typeof k === 'string') {
                    return console.log(k);
                }
                else if (Array.isArray(k)) {
                    console.log(arrayToString(k));
                }
                else if (typeof k === 'object') {
                    console.log(objectToString(k));
                }
            });
        };
    };
    CommandLineImpl = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject('NestedYargs'))
    ], CommandLineImpl);
    return CommandLineImpl;
}());
exports.CommandLineImpl = CommandLineImpl;
