"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var inversify_1 = require("inversify");
var child_process_1 = require('child_process');
var force_error_impl_1 = require("./force-error-impl");
var readlineSync = require('readline-sync');
var inpathSync = require('inpath').sync;
var pidof = require('pidof');
//noinspection JSUnusedGlobalSymbols
var SpawnImpl = (function (_super) {
    __extends(SpawnImpl, _super);
    function SpawnImpl(_commandUtil) {
        _super.call(this);
        this.commandUtil = _commandUtil;
    }
    SpawnImpl.prototype.spawnShellCommandSync = function (cmd, options, cb) {
        if (options === void 0) { options = {}; }
        if (cb === void 0) { cb = null; }
        cb = this.checkCallback(cb);
        if (this.checkForceError('spawnShellCommandSync', cb)) {
            return;
        }
        try {
            cmd = cmd || [];
            options = options || {};
            options.stdio = options.stdio || 'inherit';
            options.cwd = options.cwd || __dirname;
            this.commandUtil.log("Running '" + cmd + "' @ '" + options.cwd + "'");
            var command = cmd.shift();
            var child = child_process_1.spawnSync(command, cmd, options);
            cb(child.error, child);
        }
        catch (err) {
            cb(err, null);
        }
    };
    SpawnImpl.prototype.spawnShellCommandAsync = function (cmd, options, cb) {
        options = options || { stdio: 'pipe', cwd: '.' };
        var command = cmd.shift();
        var child = child_process_1.spawn(command, cmd, options);
        var result = '';
        child.stdout.on('data', function (data) {
            result += data.toString();
        });
        //noinspection JSUnusedLocalSymbols
        child.on('error', function (code) {
            if (cb) {
                cb(new Error('spawn error'), null);
                cb = null;
            }
        });
        //noinspection JSUnusedLocalSymbols
        child.on('close', function (code) {
            if (cb) {
                cb(null, result);
                cb = null;
            }
        });
    };
    SpawnImpl.prototype.sudoSpawn = function (cmd, cb) {
        SpawnImpl._sudoSpawn(cmd, cb);
    };
    SpawnImpl.prototype.sudoSpawnSync = function (cmd) {
        return SpawnImpl._sudoSpawnSync(cmd);
    };
    SpawnImpl._sudoSpawn = function (cmd, cb) {
        var child = SpawnImpl._sudoSpawnSync(cmd);
        child.stdout.on('data', function (data) {
            console.log(data.toString());
        });
        child.stdout.on('end', function () {
            if (cb) {
                cb();
                cb = null;
            }
        });
        child.stdout.on('close', function () {
            if (cb) {
                cb();
                cb = null;
            }
        });
        child.stdout.on('error', function () {
            if (cb) {
                cb(new Error('Something went wrong with spawn'));
                cb = null;
            }
        });
    };
    SpawnImpl._sudoSpawnSync = function (command) {
        var prompt = '#node-sudo-passwd#';
        var prompts = 0;
        var args = ['-S', '-p', prompt];
        args.push.apply(args, command);
        // The binary is the first non-dashed parameter to sudo
        var bin = command.filter(function (i) {
            return i.indexOf('-') !== 0;
        })[0];
        var spawnOptions = {
            stdio: 'pipe'
        };
        var path = process.env['PATH'].split(':');
        var sudoBin = inpathSync('sudo', path);
        var child = child_process_1.spawn(sudoBin, args, spawnOptions);
        // Wait for the sudo:d binary to start up
        function waitForStartup(err, pid) {
            if (err) {
                throw new Error("Couldn't start " + bin);
            }
            if (pid) {
                child.emit('started');
            }
            else {
                setTimeout(function () {
                    pidof(bin, waitForStartup);
                }, 100);
            }
        }
        pidof(bin, waitForStartup);
        // FIXME: Remove this handler when the child has successfully started
        child.stderr.on('data', function (data) {
            var lines = data.toString().trim().split('\n');
            lines.forEach(function (line) {
                if (line === prompt) {
                    if (++prompts > 1) {
                        // The previous entry must have been incorrect, since sudo asks again.
                        SpawnImpl.cachedPassword = null;
                    }
                    if (SpawnImpl.cachedPassword) {
                        child.stdin.write(SpawnImpl.cachedPassword + '\n');
                    }
                    else {
                        SpawnImpl.cachedPassword = SpawnImpl.cachedPassword
                            || readlineSync.question('sudo requires your password: ', { hideEchoBack: true });
                        child.stdin.write(SpawnImpl.cachedPassword + '\n');
                    }
                }
            });
        });
        return child;
    };
    SpawnImpl = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject('CommandUtil'))
    ], SpawnImpl);
    return SpawnImpl;
}(force_error_impl_1.ForceErrorImpl));
exports.SpawnImpl = SpawnImpl;
